<ul class="dropdown-menu" ng-style="{display: (isOpen && 'block') || 'none', top: position.top+'px', left: position.left+'px'}">
	<li ng-transclude></li>
	<li class="divider"></li>
	<li style="padding: 9px;">
		<span class="btn-group">
			<button type="button" class="btn btn-small btn-inverse" ng-click="today()">{{currentText}}</button>
			<button type="button" class="btn btn-small btn-info" ng-click="showWeeks = ! showWeeks" ng-class="{active: showWeeks}">{{toggleWeeksText}}</button>
			<button type="button" class="btn btn-small btn-danger" ng-click="clear()">{{clearText}}</button>
		</span>
		<button type="button" class="btn btn-small btn-success pull-right" ng-click="isOpen = false">{{closeText}}</button>
	</li>
</ul>
